import unittest
from pathlib import Path
import numpy as np

from xpydf.pdf_loader import PdfLoader

DATA = Path(__file__).parent / "test_data"


class TestPdfLoader(unittest.TestCase):
    def test_non_existent_file(self):
        with self.assertRaises(IOError):
            PdfLoader(str(DATA / "missing.pdf"))

    def test_text_extraction(self):
        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"))

        text = loader.extract_strings()

        self.assertEqual(2, len(text))

        lines = text[0].split("\n")
        self.assertEqual(6, len(lines))
        self.assertEqual("XPyDF Testing", lines[0])

    def test_page_info(self):
        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"))
        page_info = loader.extract_page_info()

        self.assertEqual(2, len(page_info))

        page = page_info[0]
        self.assertEqual(1, page["page_number"])
        self.assertEqual(612, page["width"])
        self.assertEqual(792, page["height"])

    def test_info_extraction(self):
        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"))
        page_info = loader.extract_page_info()
        images = page_info[0]["images"]

        self.assertEqual(3, len(images))

        self.assertEqual(100, images[0]["width"])
        self.assertEqual(100, images[0]["height"])

        self.assertEqual(50, images[1]["width"])
        self.assertEqual(50, images[1]["height"])

        self.assertEqual(200, images[2]["width"])
        self.assertEqual(200, images[2]["height"])
    
    def test_image_extraction(self):
        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"))
        images = loader.extract_images(1)
        
        self.assertEqual(3, len(images))
        
        self.assertEqual((100, 100, 3), images[0].shape)
        self.assertEqual((100, 100, 3), images[1].shape)
        self.assertEqual((200, 200, 3), images[2].shape)

        self.assertEqual(images[0].dtype, np.uint8)
        self.assertEqual(images[1].dtype, np.uint8)
        self.assertEqual(images[2].dtype, np.uint8)

        self.assertTrue(np.all(images[0][:,:,0] == 255))
        self.assertTrue(np.all(images[0][:,:,1] == 0))
        self.assertTrue(np.all(images[0][:,:,2] == 0))

        self.assertTrue(np.all(images[1][:,:,0] == 255))
        self.assertTrue(np.all(images[1][:,:,1] == 0))
        self.assertTrue(np.all(images[1][:,:,2] == 0))

        self.assertTrue(np.all(images[2][:,:,0] == 0))
        self.assertTrue(np.all(images[2][:,:,1] == 255))
        self.assertTrue(np.all(images[2][:,:,2] == 0))


        images = loader.extract_images(2)
        self.assertEqual(1, len(images))
        self.assertEqual((150, 150), images[0].shape)
        self.assertTrue(np.all(images[0] == 150))
    
    def test_modes(self):
        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"), mode="table")
        text = loader.extract_strings()
        self.assertEqual(2, len(text))

        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"), mode="simple")
        text = loader.extract_strings()
        self.assertEqual(2, len(text))

        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"), mode="lineprinter")
        text = loader.extract_strings()
        self.assertEqual(2, len(text))

        loader = PdfLoader(str(DATA / "xpdf_tests.pdf"), mode="physical")
        text = loader.extract_strings()
        self.assertEqual(2, len(text))
    
    def test_password(self):
        with self.assertRaises(OSError):
            PdfLoader(str(DATA / "xpdf_tests_password.pdf"))
        
        loader = PdfLoader(str(DATA / "xpdf_tests_password.pdf"), owner_password="ownerpassword")
        text = loader.extract_strings()
        self.assertEqual(1, len(text))
        
        loader = PdfLoader(str(DATA / "xpdf_tests_password.pdf"), user_password="userpassword")
        text = loader.extract_strings()
        self.assertEqual(1, len(text))
        
        loader = PdfLoader(str(DATA / "xpdf_tests_password.pdf"), owner_password="ownerpassword", user_password="userpassword")
        text = loader.extract_strings()
        self.assertEqual(1, len(text))
            



import os
from glob import glob
from pathlib import Path
import numpy as np

from setuptools import Extension, setup

python_dir = Path("src/xpydf")
xpdf_dir = Path("src/xpdf-4.04/xpdf")
splash_dir = Path("src/xpdf-4.04/splash")
fofi_dir = Path("src/xpdf-4.04/fofi")
goo_dir = Path("src/xpdf-4.04/goo")

python_src = glob(str(python_dir / "*.cc"))
splash_src = glob(str(splash_dir / "*.cc"))
fofi_src = glob(str(fofi_dir / "*.cc"))
goo_src = glob(str(goo_dir / "*.cc"))

xpdf_files = [
    "AcroForm.cc",
    "Annot.cc",
    "Array.cc",
    "BuiltinFont.cc",
    "BuiltinFontTables.cc",
    "Catalog.cc",
    "CharCodeToUnicode.cc",
    "CMap.cc",
    "Decrypt.cc",
    "Dict.cc",
    "Error.cc",
    "FontEncodingTables.cc",
    "Function.cc",
    "Gfx.cc",
    "GfxFont.cc",
    "GfxState.cc",
    "GlobalParams.cc",
    "ImageOutputDev.cc",
    "JArithmeticDecoder.cc",
    "JBIG2Stream.cc",
    "JPXStream.cc",
    "Lexer.cc",
    "Link.cc",
    "NameToCharCode.cc",
    "Object.cc",
    "OptionalContent.cc",
    "Outline.cc",
    "OutputDev.cc",
    "Page.cc",
    "Parser.cc",
    "PDF417Barcode.cc",
    "PDFDoc.cc",
    "PDFDocEncoding.cc",
    "PSTokenizer.cc",
    "SecurityHandler.cc",
    "Stream.cc",
    "TextOutputDev.cc",
    "TextString.cc",
    "UnicodeMap.cc",
    "UnicodeRemapping.cc",
    "UnicodeTypeTable.cc",
    "UTF8.cc",
    "XFAScanner.cc",
    "XRef.cc",
    "Zoox.cc",
]

xpdf_src = [str(xpdf_dir / filename) for filename in xpdf_files]

if os.name == 'nt':
    linker_libs = ['Ole32','AdvAPI32','shell32']
else:
    linker_libs = []
    
cXpdfPython = Extension(
    "cXpdfPython",
    sources=python_src + xpdf_src + splash_src + goo_src + fofi_src,
    include_dirs=[
        "src/xpdf-4.04",
        str(xpdf_dir),
        str(fofi_dir),
        str(splash_dir),
        str(goo_dir),
        str(python_dir),
        np.get_include(),
    ],
    libraries = linker_libs,
    extra_compile_args=[
        "-std=c++11",
    ],
)

setup(ext_modules=[cXpdfPython])
